@@ -32,12 +32,13 @@
 #' @param ... arguments passed onto methods.
 #' @export
 #' @author Carson Sievert
-#' @references \url{https://plot.ly/ggplot2}
+#' @references \url{https://plotly.com/ggplot2/}
 #' @seealso [plot_ly()]
 #' @examples \dontrun{
 #' # simple example
-#' ggiris <- qplot(Petal.Width, Sepal.Length, data = iris, color = Species)
-#' ggplotly(ggiris)
+#' ggpenguins <- qplot(bill_length_mm , body_mass_g, 
+#' data = palmerpenguins::penguins, color = species)
+#' ggplotly(ggpenguins)
 #'
 #' data(canada.cities, package = "maps")
 #' viz <- ggplot(canada.cities, aes(long, lat)) +
@@ -58,7 +59,7 @@
 #' demo("crosstalk-highlight-ggplotly", package = "plotly")
 #' 
 #' # client-side linked brushing in a scatterplot matrix
-#' highlight_key(iris) %>%
+#' highlight_key(palmerpenguins::penguins) %>%
 #'   GGally::ggpairs(aes(colour = Species), columns = 1:4) %>%
 #'   ggplotly(tooltip = c("x", "y", "colour")) %>%
 #'   highlight("plotly_selected")
@@ -127,7 +128,7 @@ ggplotly.ggmatrix <- function(p = ggplot2::last_plot(), width = NULL,
                titleY = TRUE, titleX = TRUE) %>% 
     hide_legend() %>%
     layout(dragmode = "select")
-  if (nchar(p$title %||% "") > 0) {
+  if (robust_nchar(p$title) > 0) {
     s <- layout(s, title = p$title)
   }
   for (i in seq_along(p$xAxisLabels)) {
@@ -175,12 +176,12 @@ gg2list <- function(p, width = NULL, height = NULL,
   
   # To convert relative sizes correctly, we use grid::convertHeight(),
   # which requires a known output (device) size.
-  dev_fun <- if (system.file(package = "Cairo") != "") {
-    Cairo::Cairo
-  } else if (capabilities("png")) {
+  dev_fun <- if (capabilities("aqua") || capabilities("png")) {
     grDevices::png
   } else if (capabilities("jpeg")) {
     grDevices::jpeg 
+  } else if (system.file(package = "Cairo") != "") {
+    Cairo::Cairo
   } else {
     stop(
       "No Cairo or bitmap device is available. Such a graphics device is required to convert sizes correctly in ggplotly().\n\n", 
@@ -214,212 +215,234 @@ gg2list <- function(p, width = NULL, height = NULL,
   # Our internal version of ggplot2::ggplot_build(). Modified from
   # https://github.com/hadley/ggplot2/blob/0cd0ba/R/plot-build.r#L18-L92
   # ------------------------------------------------------------------------
-  
-  plot <- ggfun("plot_clone")(p)
-  if (length(plot$layers) == 0) {
-    plot <- plot + geom_blank()
-  }
-  layers <- plot$layers
-  layer_data <- lapply(layers, function(y) y$layer_data(plot$data))
-  
-  # save crosstalk sets before this attribute gets squashed
-  sets <- lapply(layer_data, function(y) attr(y, "set"))
-  
-  scales <- plot$scales
-
-  # Apply function to layer and matching data
-  by_layer <- function(f) {
-    out <- vector("list", length(data))
-    for (i in seq_along(data)) {
-      out[[i]] <- f(l = layers[[i]], d = data[[i]])
+  ggplotly_build <- function(p) {
+    plot <- ggfun("plot_clone")(p)
+    
+    if (length(plot$layers) == 0) {
+      plot <- plot + geom_blank()
     }
-    out
-  }
-  
-  # ggplot2 3.1.0.9000 introduced a Layer method named setup_layer() 
-  # currently, LayerSf is the only core-ggplot2 Layer that makes use
-  # of it https://github.com/tidyverse/ggplot2/pull/2875#issuecomment-438708426
-  data <- layer_data
-  if (packageVersion("ggplot2") > "3.1.0") {
-    data <- by_layer(function(l, d) if (is.function(l$setup_layer)) l$setup_layer(d, plot) else d)
-  }
-  
-  # Initialise panels, add extra data for margins & missing facetting
-  # variables, and add on a PANEL variable to data
-  layout <- ggfun("create_layout")(plot$facet, plot$coordinates)
-  data <- layout$setup(data, plot$data, plot$plot_env)
-  
-  # save the domain of the group for display in tooltips
-  groupDomains <- Map(function(x, y) {
-    aes_g <- y$mapping[["group"]] %||% plot$mapping[["group"]]
-    tryNULL(rlang::eval_tidy(aes_g, x))
-  }, data, layers)
-  
-  # for simple (StatIdentity) geoms, add crosstalk key to aes mapping
-  # (effectively adding it as a group)
-  # later on, for more complicated geoms (w/ non-trivial summary statistics),
-  # we construct a nested key mapping (within group)
-  layers <- Map(function(x, y) {
-    if (crosstalk_key() %in% names(y) && !"key" %in% names(x[["mapping"]]) && 
-        inherits(x[["stat"]], "StatIdentity")) {
-      x[["mapping"]] <- c(x[["mapping"]], key = as.name(crosstalk_key()))
+    layers <- plot$layers
+    layer_data <- lapply(layers, function(y) y$layer_data(plot$data))
+    
+    # save crosstalk sets before this attribute gets squashed
+    sets <- lapply(layer_data, function(y) attr(y, "set"))
+    
+    scales <- plot$scales
+    
+    # Apply function to layer and matching data
+    by_layer <- function(f) {
+      out <- vector("list", length(data))
+      for (i in seq_along(data)) {
+        out[[i]] <- f(l = layers[[i]], d = data[[i]])
+      }
+      out
     }
-    x
-  }, layers, layer_data)
-  
-  # Compute aesthetics to produce data with generalised variable names
-  data <- by_layer(function(l, d) l$compute_aesthetics(d, plot))
-  
-  # add frame to group if it exists
-  data <- lapply(data, function(d) { 
-    if (!"frame" %in% names(d)) return(d)
-    d$group <- with(d, paste(group, frame, sep = "-"))
-    d
-  })
-  
-  # The computed aesthetic codes the groups as integers
-  # Here we build a map each of the integer values to the group label
-  group_maps <- Map(function(x, y) {
-    tryCatch({
-      x_group <- x[["group"]]
-      names(x_group) <- y
-      x_group <- x_group[!duplicated(x_group)]
-      x_group
-    }, error = function(e) NULL
+    
+    # ggplot2 3.1.0.9000 introduced a Layer method named setup_layer() 
+    # currently, LayerSf is the only core-ggplot2 Layer that makes use
+    # of it https://github.com/tidyverse/ggplot2/pull/2875#issuecomment-438708426
+    data <- layer_data
+    if (packageVersion("ggplot2") > "3.1.0") {
+      data <- by_layer(function(l, d) if (is.function(l$setup_layer)) l$setup_layer(d, plot) else d)
+    }
+    
+    # Initialise panels, add extra data for margins & missing facetting
+    # variables, and add on a PANEL variable to data
+    layout <- ggfun("create_layout")(plot$facet, plot$coordinates)
+    data <- layout$setup(data, plot$data, plot$plot_env)
+    
+    # save the domain of the group for display in tooltips
+    groupDomains <- Map(function(x, y) {
+      aes_g <- y$mapping[["group"]] %||% plot$mapping[["group"]]
+      tryNULL(rlang::eval_tidy(aes_g, x))
+    }, data, layers)
+    
+    # for simple (StatIdentity) geoms, add crosstalk key to aes mapping
+    # (effectively adding it as a group)
+    # later on, for more complicated geoms (w/ non-trivial summary statistics),
+    # we construct a nested key mapping (within group)
+    layers <- Map(function(x, y) {
+      if (crosstalk_key() %in% names(y) && !"key" %in% names(x[["mapping"]]) && 
+          inherits(x[["stat"]], "StatIdentity")) {
+        x[["mapping"]] <- c(x[["mapping"]], key = as.name(crosstalk_key()))
+      }
+      x
+    }, layers, layer_data)
+    
+    # Compute aesthetics to produce data with generalised variable names
+    data <- by_layer(function(l, d) l$compute_aesthetics(d, plot))
+    
+    # add frame to group if it exists
+    data <- lapply(data, function(d) { 
+      if (!"frame" %in% names(d)) return(d)
+      d$group <- with(d, paste(group, frame, sep = "-"))
+      d
+    })
+    
+    # The computed aesthetic codes the groups as integers
+    # Here we build a map each of the integer values to the group label
+    group_maps <- Map(function(x, y) {
+      tryCatch({
+        x_group <- x[["group"]]
+        names(x_group) <- y
+        x_group <- x_group[!duplicated(x_group)]
+        x_group
+      }, error = function(e) NULL
+      )
+    }, data, groupDomains)
+    
+    # Before mapping x/y position, save the domain (for discrete scales)
+    # to display in tooltip.
+    data <- lapply(data, function(d) {
+      d[["x_plotlyDomain"]] <- d[["x"]]
+      d[["y_plotlyDomain"]] <- d[["y"]]
+      d
+    })
+    
+    # Transform all scales
+    data <- lapply(data, ggfun("scales_transform_df"), scales = scales)
+    
+    # Map and train positions so that statistics have access to ranges
+    # and all positions are numeric
+    scale_x <- function() scales$get_scales("x")
+    scale_y <- function() scales$get_scales("y")
+    
+    layout$train_position(data, scale_x(), scale_y())
+    
+    data <- layout$map_position(data)
+    
+    # build a mapping between group and key
+    # if there are multiple keys within a group, the key is a list-column
+    reComputeGroup <- function(x, layer = NULL) {
+      # 1-to-1 link between data & visual marks -- group == key
+      if (inherits(layer$geom, "GeomDotplot")) {
+        x <- split(x, x[["PANEL"]])
+        x <- lapply(x, function(d) { 
+          d[["group"]] <- do.call("order", d[c("x", "group")]) 
+          d 
+        })
+        x <- dplyr::bind_rows(x)
+      }
+      if (inherits(layer$geom, "GeomSf")) {
+        x <- split(x, x[["PANEL"]])
+        x <- lapply(x, function(d) { 
+          d[["group"]] <- seq_len(nrow(d))
+          d 
+        })
+        # I think this is safe?
+        x <- suppressWarnings(dplyr::bind_rows(x))
+      }
+      x
+    }
+    
+    nestedKeys <- Map(function(x, y, z) { 
+      key <- y[[crosstalk_key()]]
+      if (is.null(key) || inherits(z[["stat"]], "StatIdentity")) return(NULL)
+      x <- reComputeGroup(x, z)
+      tib <- tibble::as_tibble(x[c("PANEL", "group")])
+      tib[["key"]] <- key
+      nested <- tidyr::nest(tib, key, .key = key)
+      # reduce the dimensions of list column elements from 2 to 1
+      nested$key <- lapply(nested$key, function(x) x[[1]])
+      nested
+    }, data, layer_data, layers)
+    
+    # for some geoms (e.g. boxplots) plotly.js needs the "pre-statistics" data
+    # we also now provide the option to return one of these two
+    prestats_data <- data
+    data <- by_layer(function(l, d) l$compute_statistic(d, layout))
+    data <- by_layer(function(l, d) l$map_statistic(d, plot))
+    
+    # Make sure missing (but required) aesthetics are added
+    ggfun("scales_add_missing")(plot, c("x", "y"), plot$plot_env)
+    
+    # Reparameterise geoms from (e.g.) y and width to ymin and ymax
+    data <- by_layer(function(l, d) l$compute_geom_1(d))
+    
+    # compute_geom_1 can reorder the rows from `data`, making groupDomains
+    # invalid. We rebuild groupDomains based on the current `data` and the
+    # group map we built before.
+    groupDomains <- Map(function(x, y) {
+      tryCatch({
+        names(y)[match(x$group, y)]
+      }, error = function(e) NULL
+      )
+    }, data, group_maps)
+    
+    # there are some geoms (e.g. geom_dotplot()) where attaching the key 
+    # before applying the statistic can cause problems, but there is still a 
+    # 1-to-1 corresponding between graphical marks and 
+    
+    # Apply position adjustments
+    data <- by_layer(function(l, d) l$compute_position(d, layout))
+    
+    # Reset position scales, then re-train and map.  This ensures that facets
+    # have control over the range of a plot: is it generated from what's
+    # displayed, or does it include the range of underlying data
+    layout$reset_scales()
+    layout$train_position(data, scale_x(), scale_y())
+    layout$setup_panel_params()
+    data <- layout$map_position(data)
+    
+    # Train and map non-position scales
+    npscales <- scales$non_position_scales()
+    if (npscales$n() > 0) {
+      lapply(data, ggfun("scales_train_df"), scales = npscales)
+      # this for loop is unique to plotly -- it saves the "domain"
+      # of each non-positional scale for display in tooltips
+      for (sc in npscales$scales) {
+        data <- lapply(data, function(d) {
+          # scale may not be relevant for every layer data
+          if (any(names(d) %in% sc$aesthetics)) {
+            d[paste0(sc$aesthetics, "_plotlyDomain")] <- d[sc$aesthetics]
+          }
+          d
+        })
+      }
+      data <- lapply(data, ggfun("scales_map_df"), scales = npscales)
+    }
+    
+    # Fill in defaults etc.
+    data <- by_layer(function(l, d) l$compute_geom_2(d))
+    
+    # Let layer stat have a final say before rendering
+    data <- by_layer(function(l, d) l$finish_statistics(d))
+    
+    # Let Layout modify data before rendering
+    data <- layout$finish_data(data)
+    
+    # if necessary, attach key
+    data <- Map(function(x, y, z) { 
+      if (!length(y)) return(x)
+      x <- reComputeGroup(x, z)
+      # dplyr issue??? https://github.com/tidyverse/dplyr/issues/2701
+      attr(y$group, "n") <- NULL
+      suppressMessages(dplyr::left_join(x, y))
+    }, data, nestedKeys, layers)
+    
+    structure(
+      list(
+        data = data, layout = layout, plot = plot, 
+        env = environment()
+      ), 
+      class = "ggplot_built"
     )
-  }, data, groupDomains)
-  
-  # Before mapping x/y position, save the domain (for discrete scales)
-  # to display in tooltip.
-  data <- lapply(data, function(d) {
-    d[["x_plotlyDomain"]] <- d[["x"]]
-    d[["y_plotlyDomain"]] <- d[["y"]]
-    d
-  })
-
-  # Transform all scales
-  data <- lapply(data, ggfun("scales_transform_df"), scales = scales)
-  
-  # Map and train positions so that statistics have access to ranges
-  # and all positions are numeric
-  scale_x <- function() scales$get_scales("x")
-  scale_y <- function() scales$get_scales("y")
-  
-  layout$train_position(data, scale_x(), scale_y())
-  
-  data <- layout$map_position(data)
-  
-  # build a mapping between group and key
-  # if there are multiple keys within a group, the key is a list-column
-  reComputeGroup <- function(x, layer = NULL) {
-    # 1-to-1 link between data & visual marks -- group == key
-    if (inherits(layer$geom, "GeomDotplot")) {
-      x <- split(x, x[["PANEL"]])
-      x <- lapply(x, function(d) { 
-        d[["group"]] <- do.call("order", d[c("x", "group")]) 
-        d 
-      })
-      x <- dplyr::bind_rows(x)
-    }
-    if (inherits(layer$geom, "GeomSf")) {
-      x <- split(x, x[["PANEL"]])
-      x <- lapply(x, function(d) { 
-        d[["group"]] <- seq_len(nrow(d))
-        d 
-      })
-      # I think this is safe?
-      x <- suppressWarnings(dplyr::bind_rows(x))
-    }
-    x
   }
   
-  nestedKeys <- Map(function(x, y, z) { 
-    key <- y[[crosstalk_key()]]
-    if (is.null(key) || inherits(z[["stat"]], "StatIdentity")) return(NULL)
-    x <- reComputeGroup(x, z)
-    tib <- tibble::as_tibble(x[c("PANEL", "group")])
-    tib[["key"]] <- key
-    nested <- tidyr::nest(tib, key, .key = key)
-    # reduce the dimensions of list column elements from 2 to 1
-    nested$key <- lapply(nested$key, function(x) x[[1]])
-    nested
-  }, data, layer_data, layers)
-  
-  # for some geoms (e.g. boxplots) plotly.js needs the "pre-statistics" data
-  # we also now provide the option to return one of these two
-  prestats_data <- data
-  data <- by_layer(function(l, d) l$compute_statistic(d, layout))
-  data <- by_layer(function(l, d) l$map_statistic(d, plot))
-  
-  # Make sure missing (but required) aesthetics are added
-  ggfun("scales_add_missing")(plot, c("x", "y"), plot$plot_env)
-  
-  # Reparameterise geoms from (e.g.) y and width to ymin and ymax
-  data <- by_layer(function(l, d) l$compute_geom_1(d))
-
-  # compute_geom_1 can reorder the rows from `data`, making groupDomains
-  # invalid. We rebuild groupDomains based on the current `data` and the
-  # group map we built before.
-  groupDomains <- Map(function(x, y) {
-    tryCatch({
-      names(y)[match(x$group, y)]
-    }, error = function(e) NULL
-    )
-  }, data, group_maps)
-  
-  # there are some geoms (e.g. geom_dotplot()) where attaching the key 
-  # before applying the statistic can cause problems, but there is still a 
-  # 1-to-1 corresponding between graphical marks and 
-
-  # Apply position adjustments
-  data <- by_layer(function(l, d) l$compute_position(d, layout))
-  
-  # Reset position scales, then re-train and map.  This ensures that facets
-  # have control over the range of a plot: is it generated from what's
-  # displayed, or does it include the range of underlying data
-  layout$reset_scales()
-  layout$train_position(data, scale_x(), scale_y())
-  layout$setup_panel_params()
-  data <- layout$map_position(data)
-  
-  # Train and map non-position scales
-  npscales <- scales$non_position_scales()
-  if (npscales$n() > 0) {
-    lapply(data, ggfun("scales_train_df"), scales = npscales)
-    # this for loop is unique to plotly -- it saves the "domain"
-    # of each non-positional scale for display in tooltips
-    for (sc in npscales$scales) {
-      data <- lapply(data, function(d) {
-        # scale may not be relevant for every layer data
-        if (any(names(d) %in% sc$aesthetics)) {
-          d[paste0(sc$aesthetics, "_plotlyDomain")] <- d[sc$aesthetics]
-        }
-        d
-      })
-    }
-    data <- lapply(data, ggfun("scales_map_df"), scales = npscales)
+  # Allow thematic to add new defaults to the plot object based on it's theme
+  built <- if (isNamespaceLoaded("thematic")) {
+    tns <- asNamespace("thematic")
+    tns$ggthematic_build(p, ggplotly_build, tns$thematic_get_theme(resolve = TRUE))
+  } else {
+    ggplotly_build(p)
   }
   
-  # Fill in defaults etc.
-  data <- by_layer(function(l, d) l$compute_geom_2(d))
-  
-  # Let layer stat have a final say before rendering
-  data <- by_layer(function(l, d) l$finish_statistics(d))
-  
-  # Let Layout modify data before rendering
-  data <- layout$finish_data(data)
-  
-  # ------------------------------------------------------------------------
-  # end of ggplot_build()
-  # ------------------------------------------------------------------------
-  # if necessary, attach key
-  data <- Map(function(x, y, z) { 
-    if (!length(y)) return(x)
-    x <- reComputeGroup(x, z)
-    # dplyr issue??? https://github.com/tidyverse/dplyr/issues/2701
-    attr(y$group, "n") <- NULL
-    suppressMessages(dplyr::left_join(x, y))
-  }, data, nestedKeys, layers)
+  # Assign all the objects available to ggplotly_build() to this functions environment
+  built_env <- built$env
+  envir <- environment()
+  for (var in ls(built_env)) {
+    assign(var, built_env[[var]], envir = envir)
+  }
   
   # initiate plotly.js layout with some plot-wide theming stuff
   theme <- ggfun("plot_theme")(plot)
@@ -436,7 +459,7 @@ gg2list <- function(p, width = NULL, height = NULL,
     font = text2font(theme$text)
   )
   # main plot title
-  if (nchar(plot$labels$title %||% "") > 0) {
+  if (robust_nchar(plot$labels$title) > 0) {
     gglayout$title <- list(
       text = faced(plot$labels$title, theme$plot.title$face),
       font = text2font(theme$plot.title),
@@ -495,7 +518,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       )
     }
     # anchor X axis to the lowest plot in its column
-    layout$layout <- dplyr::group_by_(layout$layout, "xaxis")
+    layout$layout <- dplyr::group_by(layout$layout, !!rlang::sym("xaxis"))
     layout$layout <-  dplyr::mutate(layout$layout, xanchor = max(as.integer(yaxis)))
   }
   layout$layout <- as.data.frame(layout$layout)
@@ -567,7 +590,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       # allocate enough space for the _longest_ text label
       axisTextX <- theme[["axis.text.x"]] %||% theme[["axis.text"]]
       labz <- unlist(lapply(layout$panel_params, function(pp) { pp[["x"]]$get_labels %()% pp$x.labels }))
-      lab <- labz[which.max(nchar(labz))]
+      lab <- longest_element(labz)
       panelMarginY <- panelMarginY + axisTicksX +
         bbox(lab, axisTextX$angle, unitConvert(axisTextX, "npc", "height"))[["height"]]
     }
@@ -579,7 +602,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       # allocate enough space for the _longest_ text label
       axisTextY <- theme[["axis.text.y"]] %||% theme[["axis.text"]]
       labz <- unlist(lapply(layout$panel_params, function(pp) { pp[["y"]]$get_labels %()% pp$y.labels }))
-      lab <- labz[which.max(nchar(labz))]
+      lab <- longest_element(labz)
       panelMarginX <- panelMarginX + axisTicksY +
         bbox(lab, axisTextY$angle, unitConvert(axisTextY, "npc", "width"))[["width"]]
     }
@@ -806,7 +829,9 @@ gg2list <- function(p, width = NULL, height = NULL,
       
       # do some stuff that should be done once for the entire plot
       if (i == 1) {
-        axisTickText <- axisObj$ticktext[which.max(nchar(axisObj$ticktext))]
+        # Split ticktext elements by "\n"  to account for linebreaks
+        axisTickText <- strsplit(as.character(axisObj$ticktext), split = "\n", fixed = TRUE)
+        axisTickText <- longest_element(unlist(axisTickText))
         side <- if (xy == "x") "b" else "l"
         # account for axis ticks, ticks text, and titles in plot margins
         # (apparently ggplot2 doesn't support axis.title/axis.text margins)
@@ -814,7 +839,7 @@ gg2list <- function(p, width = NULL, height = NULL,
           bbox(axisTickText, axisObj$tickangle, axisObj$tickfont$size)[[type]] +
           bbox(axisTitleText, axisTitle$angle, unitConvert(axisTitle, "pixels", type))[[type]]
         
-        if (nchar(axisTitleText) > 0) {
+        if (robust_nchar(axisTitleText) > 0) {
           axisTextSize <- unitConvert(axisText, "npc", type)
           axisTitleSize <- unitConvert(axisTitle, "npc", type)
           offset <-
@@ -836,7 +861,7 @@ gg2list <- function(p, width = NULL, height = NULL,
           }
           # facets have multiple axis objects, but only one title for the plot,
           # so we empty the titles and try to draw the title as an annotation
-          if (nchar(axisTitleText) > 0) {
+          if (robust_nchar(axisTitleText) > 0) {
             # npc is on a 0-1 scale of the _entire_ device,
             # but these units _should_ be wrt to the plotting region
             # multiplying the offset by 2 seems to work, but this is a terrible hack
@@ -873,7 +898,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       )
       if (is_blank(theme[["strip.text.x"]])) col_txt <- ""
       if (inherits(plot$facet, "FacetGrid") && lay$ROW != 1) col_txt <- ""
-      if (nchar(col_txt) > 0) {
+      if (robust_nchar(col_txt) > 0) {
         col_lab <- make_label(
           col_txt, x = mean(xdom), y = max(ydom),
           el = theme[["strip.text.x"]] %||% theme[["strip.text"]],
@@ -890,7 +915,7 @@ gg2list <- function(p, width = NULL, height = NULL,
       )
       if (is_blank(theme[["strip.text.y"]])) row_txt <- ""
       if (inherits(plot$facet, "FacetGrid") && lay$COL != nCols) row_txt <- ""
-      if (nchar(row_txt) > 0) {
+      if (robust_nchar(row_txt) > 0) {
         row_lab <- make_label(
           row_txt, x = max(xdom), y = mean(ydom),
           el = theme[["strip.text.y"]] %||% theme[["strip.text"]],
@@ -1088,7 +1113,7 @@ gg2list <- function(p, width = NULL, height = NULL,
   # translate group aesthetics to data attributes
   return_dat <- Map(function(x, y) {
     if (is.null(y[["group"]])) return(x)
-    dplyr::group_by_(x, y[["group"]])
+    dplyr::group_by(x, !!rlang::as_quosure(y[["group"]]))
   }, return_dat, mappingFormulas)
   
   # don't need to add group as an attribute anymore
@@ -1107,6 +1132,10 @@ gg2list <- function(p, width = NULL, height = NULL,
 }
 
 
+# Due to the non-standard use of assign() in g2list() (above)
+utils::globalVariables(c("groupDomains", "layers", "prestats_data", "scales", "sets"))
+
+
 #-----------------------------------------------------------------------------
 # ggplotly 'utility' functions
 #-----------------------------------------------------------------------------
@@ -1180,7 +1209,7 @@ is_blank <- function(x) {
 # given text, and x/y coordinates on 0-1 scale,
 # convert ggplot2::element_text() to plotly annotation
 make_label <- function(txt = "", x, y, el = ggplot2::element_text(), ...) {
-  if (is_blank(el) || is.null(txt) || nchar(txt) == 0 || length(txt) == 0) {
+  if (is_blank(el) || is.null(txt) || robust_nchar(txt) == 0 || length(txt) == 0) {
     return(NULL)
   }
   angle <- el$angle %||% 0
@@ -1215,9 +1244,9 @@ has_facet <- function(x) {
 
 bbox <- function(txt = "foo", angle = 0, size = 12) {
   # assuming the horizontal size of a character is roughly half of the vertical
-  n <- nchar(txt)
+  n <- robust_nchar(txt)
   if (sum(n) == 0) return(list(height = 0, width = 0))
-  w <- size * (nchar(txt) / 2)
+  w <- size * (robust_nchar(txt) / 2)
   angle <- abs(angle %||% 0)
   # do the sensible thing in the majority of cases
   if (angle == 0) return(list(height = size, width = w))
@@ -1234,12 +1263,31 @@ bbox <- function(txt = "foo", angle = 0, size = 12) {
 text2font <- function(x = ggplot2::element_text(), type = "height") {
   list(
     color = toRGB(x$colour),
-    family = x$family,
+    family = font_family(x$family),
     # TODO: what about the size of vertical text?
     size = unitConvert(grid::unit(x$size %||% 0, "points"), "pixels", type)
   )
 }
 
+# Replace a default font family, "", with thematic's font option (if set)
+font_family <- function(family = "") {
+  if (!identical(family, "")) {
+    return(family)
+  }
+  if (!isNamespaceLoaded("thematic")) {
+    return("")
+  }
+  font <- asNamespace("thematic")$thematic_get_option("font", resolve = TRUE)
+  if (!length(font)) {
+    return("")
+  }
+  # font$families is a vector of families, but font.family wants to be a 
+  # string (like CSS font-family), so make sure the names are unquoted, 
+  # then quote them
+  families <- sub("'$", "", sub("^'", "", font$families))
+  sprintf("'%s'", paste(families, collapse = "', '"))
+}
+
 # wrap text in bold/italics according to the text "face"
 faced <- function(txt, face = "plain") {
   if (is.null(face)) face <- "plain"
