@@ -139,7 +139,7 @@ crosstalk_key <- function() ".crossTalkKey"
 # arrange data if the vars exist, don't throw error if they don't
 arrange_safe <- function(data, vars) {
   vars <- vars[vars %in% names(data)]
-  if (length(vars)) dplyr::arrange_(data, .dots = vars) else data
+  if (length(vars)) dplyr::arrange(data, !!!rlang::syms(vars)) else data
 }
 
 is_mapbox <- function(p) {
@@ -521,7 +521,7 @@ verify_attr <- function(proposed, schema, layoutAttr = FALSE) {
     
     # do the same for "sub-attributes"
     if (identical(role, "object") && is.recursive(proposed[[attr]])) {
-      proposed[[attr]] <- verify_attr(proposed[[attr]], schema[[attr]], layoutAttr = layoutAttr)
+      proposed[[attr]] <- verify_attr(proposed[[attr]], attrSchema, layoutAttr = layoutAttr)
     }
   }
   
@@ -585,7 +585,7 @@ verify_type <- function(trace) {
     message(
       "No ", trace$type, " mode specifed:\n",
       "  Setting the mode to markers\n",
-      "  Read more about this attribute -> https://plot.ly/r/reference/#scatter-mode"
+      "  Read more about this attribute -> https://plotly.com/r/reference/#scatter-mode"
     )
     trace$mode <- "markers"
   }
@@ -596,7 +596,7 @@ relay_type <- function(type) {
   message(
     "No trace type specified:\n", 
     "  Based on info supplied, a '", type, "' trace seems appropriate.\n",
-    "  Read more about this trace type -> https://plot.ly/r/reference/#", type
+    "  Read more about this trace type -> https://plotly.com/r/reference/#", type
   )
   type
 }
@@ -609,12 +609,14 @@ translate_linebreaks <- function(p) {
     typ <- typeof(a)
     if (typ == "list") {
       # retain the class of list elements 
-      # which important for many things, such as colorbars
+      # which is important for many things, such as colorbars
       a[] <- lapply(a, recurse)
     } else if (typ == "character" && !inherits(a, "JS_EVAL")) {
       attrs <- attributes(a)
       a <- gsub("\n", br(), a, fixed = TRUE)
       attributes(a) <- attrs
+    } else if (is.factor(a)) {
+      levels(a) <- gsub("\n", br(), levels(a), fixed = TRUE)
     }
     a
   }
@@ -1019,7 +1021,8 @@ try_file <- function(f, what) {
 # preferred defaults for toJSON mapping
 to_JSON <- function(x, ...) {
   jsonlite::toJSON(x, digits = 50, auto_unbox = TRUE, force = TRUE,
-                   null = "null", na = "null", ...)
+                   null = "null", na = "null", 
+                   time_format = "%Y-%m-%d %H:%M:%OS6",  ...)
 }
 
 # preferred defaults for toJSON mapping
@@ -1135,7 +1138,40 @@ try_library <- function(pkg, fun = NULL) {
        "Please install and try again.", call. = FALSE)
 }
 
+# a la shiny:::is_available
+is_available <- function(package, version = NULL) {
+  installed <- nzchar(system.file(package = package))
+  if (is.null(version)) {
+    return(installed)
+  }
+  installed && isTRUE(utils::packageVersion(package) >= version)
+}
+
 # similar logic to rstudioapi::isAvailable()
 is_rstudio <- function() {
   identical(.Platform$GUI, "RStudio")
 }
+
+# nchar() needs a non-empty character vector; sometimes x will be a
+# factor, or an empty vector.
+robust_nchar <- function(x, ...) {
+  if (length(x)) nchar(as.character(x), ...)
+  else 0
+}
+
+# Extract longest element, or blank if none
+longest_element <- function(x) {
+  if (length(x))
+    x[which.max(robust_nchar(x))]
+  else
+    ""
+}
+
+# A dplyr::group_by wrapper for the add argument
+group_by_add <- function(..., add = TRUE) {
+  if (packageVersion('dplyr') >= '1.0') {
+    dplyr::group_by(...,  .add = add)
+  } else {
+    dplyr::group_by(...,  add = add)
+  } 
+}
